/*
 * Hive Video Hook
 */
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif


#include "framehook.h"
#include "swscale.h"

#ifdef __cplusplus
}
#endif


#include <hive_lib.h>

#ifdef __cplusplus
extern "C" {
#endif

static int sws_flags = SWS_BICUBIC;

typedef struct {
    // This vhook first converts frame to YUV ...
    struct SwsContext *toYUV_convert_ctx;

    // ... and later converts back frame from YUV to initial format
    struct SwsContext *fromYUV_convert_ctx;

} ContextInfo;

int HiveConfigure(void **ctxp, int argc, char *argv[])
{
  int ret = hive_init(argc,argv);

  if (ret==0)
    {
      *ctxp = av_mallocz(sizeof(ContextInfo));
    }
  else
    {
      av_log(NULL, AV_LOG_ERROR,
	     "Invalid arguments passed to Hive vhook\n");
    }

    return ret;
}


void HiveRelease(void *ctx)
{
    hive_finishup();

    ContextInfo *ci;
    ci = (ContextInfo *) ctx;

    if (ctx) {
        sws_freeContext(ci->toYUV_convert_ctx);
        sws_freeContext(ci->fromYUV_convert_ctx);
        av_free(ctx);
    }
}


void HiveProcess(void *ctx, AVPicture *picture, enum PixelFormat pix_fmt, 
		 double frame_rate, int width, int height, int64_t pts)
{
    ContextInfo *ci = (ContextInfo *) ctx;
    void *buf = 0;
    AVPicture picture1;
    AVPicture *pict = picture;

    (void) ci;

    if (pix_fmt != PIX_FMT_YUV420P) {
        int size;

        size = avpicture_get_size(PIX_FMT_YUV420P, width, height);
        buf = av_malloc(size);

        avpicture_fill(&picture1, (uint8_t*)buf, PIX_FMT_YUV420P, width, height);

        // if we already got a SWS context, let's realloc if is not re-useable
        ci->toYUV_convert_ctx = sws_getCachedContext(ci->toYUV_convert_ctx,
                                    width, height, pix_fmt,
                                    width, height, PIX_FMT_YUV420P,
                                    sws_flags, NULL, NULL, NULL);
        if (ci->toYUV_convert_ctx == NULL) {
            av_log(NULL, AV_LOG_ERROR,
                   "Cannot initialize the toYUV conversion context\n");
            return;
        }
	// img_convert parameters are          2 first destination, then 4 source
	// sws_scale   parameters are context, 4 first source,      then 2 destination
        sws_scale(ci->toYUV_convert_ctx,
            picture->data, picture->linesize, 0, height,
            picture1.data, picture1.linesize);

        pict = &picture1;
    }

    /* Insert filter code here */

    /*
    fprintf(stderr,"**** data[0] linesize = %d\n",pict->linesize[0]);
    fprintf(stderr,"**** data[1] linesize = %d\n",pict->linesize[1]);
    fprintf(stderr,"**** data[2] linesize = %d\n",pict->linesize[2]);
    fprintf(stderr,"**** data[3] linesize = %d\n",pict->linesize[3]);
    */


    hive_process_yuv420_frame(pict->data,frame_rate,width,height);


    if (pix_fmt != PIX_FMT_YUV420P) {
        ci->fromYUV_convert_ctx = sws_getCachedContext(ci->fromYUV_convert_ctx,
                                        width, height, PIX_FMT_YUV420P,
                                        width, height, pix_fmt,
                                        sws_flags, NULL, NULL, NULL);
        if (ci->fromYUV_convert_ctx == NULL) {
            av_log(NULL, AV_LOG_ERROR,
                   "Cannot initialize the fromYUV conversion context\n");
            return;
        }
	// img_convert parameters are          2 first destination, then 4 source
	// sws_scale   parameters are context, 4 first source,      then 2 destination
        sws_scale(ci->fromYUV_convert_ctx,
		  picture1.data, picture1.linesize, 0, height,
		  picture->data, picture->linesize);
    }

    av_free(buf);
}

#ifdef __cplusplus
}
#endif
