/*
 * Video processing hooks
 * Copyright (c) 2000, 2001 Fabrice Bellard.
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include <errno.h>
#include "config.h"
#include "avformat.h"
#include "framehook.h"

#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif


typedef struct _HiveHookEntry {
    void *ctx;
} HiveHookEntry;

HiveHookEntry* only_hook = NULL;

double hive_frame_rate = 25.0;


void hive_hook_set_framerate(int num, int den)
{
  hive_frame_rate = ((double)num)/(double)den;
}


/* Returns 0 on OK */
int hive_hook_add(int argc, char *argv[])
{
#ifdef CONFIG_VHOOK
    HiveHookEntry *fhe;

    if (argc < 1) {
        return ENOENT;
    }

    fhe = av_mallocz(sizeof(*fhe));
    if (!fhe) {
        return AVERROR(ENOMEM);
    }


    if (argc > 1 ) {
        if (HiveConfigure(&fhe->ctx, argc, argv)) {
            av_log(NULL, AV_LOG_ERROR, "Failed to Configure %s\n", argv[0]);
            return AVERROR(EINVAL);
        }
	only_hook = fhe;
    }


    return 0;
#else
    av_log(NULL, AV_LOG_ERROR, "Video hooking not compiled into this version\n");
    return 1;
#endif
}

void hive_hook_process(AVPicture *pict, enum PixelFormat pix_fmt, int width, int height, int64_t pts)
{
    if (only_hook) {
      HiveProcess(only_hook->ctx, pict, pix_fmt, hive_frame_rate, width, height, pts);
    }
}

void hive_hook_release(void)
{

  HiveRelease(only_hook->ctx);
  av_free(only_hook);
  only_hook = NULL;

}


