#include <string.h>
#include <stdio.h>
#include <math.h>
#include "ansesvidseg_encoder.h"

extern void Fatal(const char *fmt,...);

AnsesVidseg_Encoder::AnsesVidseg_Encoder(char *opfilename, char *vidfilename, 
			       char *keyframedir)
{
  strcpy(vidseg_opfilename, opfilename);
  strcpy(video_filename, vidfilename);

  if ((fvs = fopen(vidseg_opfilename, "w")) == NULL)
    Fatal("Failed to open vidseg output file \"%s\" for writing", vidseg_opfilename);
  
  //XML headers
  char *tailptr = strrchr(video_filename, '/');
  tailptr++;
  if (!tailptr) 
    tailptr = video_filename;
  fprintf(fvs, "VideoSegment - [0,");

  //set up keyframe filename
  kfcount = 0;
  dokeyframes = 0;
  if (keyframedir)
    {
      sprintf(kfbase, "%s/%s", keyframedir, tailptr);
      *(strrchr(kfbase, '.')) = '\0';
      dokeyframes = 1;
    }
  doneEnd = 0;
}

AnsesVidseg_Encoder::~AnsesVidseg_Encoder()
{
  fprintf(fvs, "%f]\n", currTimestamp);
  fclose(fvs);
}

void AnsesVidseg_Encoder::OutputFrame(frame* vidframe)
{
  currTimestamp = vidframe->m_dTimestamp;
  if (doneEnd)
    {
      fprintf(fvs, "VideoSegment - [%f,", currTimestamp);    
      doneEnd = 0;
    }
  if ((vidframe->transType == CUT) || (vidframe->transType == GRADUAL))
    {
      fprintf(fvs, "%f]\n", currTimestamp);
      doneEnd = 1;
    }


  fflush(fvs); // until end bug is fixed

  if (vidframe->isKeyFrame && dokeyframes)
    WriteKeyFrame(vidframe, kfbase, kfcount++);
}


//nasty global stuff for libjpeg
struct my_error_mgr{
  struct jpeg_error_mgr pub;
  jmp_buf setjmp_buffer;
};

typedef struct my_error_mgr * my_error_ptr;

METHODDEF(void) my_error_exit(j_common_ptr cinfo)
{
  my_error_ptr myerr = (my_error_ptr) cinfo->err;
  (*cinfo->err->output_message) (cinfo);
  longjmp(myerr->setjmp_buffer, 1);
}

void AnsesVidseg_Encoder::WriteKeyFrame(frame *vidframe, char *filenamebase,
				   int shotno)
{
  char filename[1024];
  sprintf(filename, "%s_%.4i.jpg", filenamebase, shotno);

  int iSrcWidth=vidframe->m_iWidth;
  int iSrcHeight=vidframe->m_iHeight;

  //Scale the Y buffer
  /*
  unsigned char *pcYbuffer = new unsigned char[(iSrcWidth * iSrcHeight) / 4];
  for (int i=0; i<iSrcWidth/2; i++)
    for (int j=0; j<iSrcHeight/2; j++)
      {
	int av = (
	  vidframe->m_pcImage[2*i + 2*j*iSrcWidth] +
	  vidframe->m_pcImage[2*i + (2*j+1)*iSrcWidth] +
	  vidframe->m_pcImage[2*i+1 + 2*j*iSrcWidth] +
	  vidframe->m_pcImage[2*i+1 + (2*j+1)*iSrcWidth]) / 4;
	pcYbuffer[i + j*(iSrcWidth/2)] = (unsigned char)av;
      }
  */
  JSAMPARRAY component_pointer[3];
  JSAMPROW y_row_pointer[16]; 
  JSAMPROW u_row_pointer[16]; 
  JSAMPROW v_row_pointer[16]; 
  
  unsigned char *y, *u, *v;
  y=vidframe->m_pcImage;//pcYbuffer;
  u=vidframe->m_pcImage + (vidframe->m_iWidth * vidframe->m_iHeight);
  v=u + (vidframe->m_iWidth * vidframe->m_iHeight /4);

  struct jpeg_compress_struct cinfo;
  struct my_error_mgr jerr;
  FILE *outfile;

  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = my_error_exit;
  if (setjmp(jerr.setjmp_buffer))
    {
      //JPEG code has signalled an error
      jpeg_destroy_compress(&cinfo);
      return;
    }
  jpeg_create_compress(&cinfo);

  if ((outfile = fopen(filename, "wb")) == NULL)
    {
      printf("Failed to create keyframe at %s\n", filename);
      return;
    }
  jpeg_stdio_dest(&cinfo, outfile);
  cinfo.image_width = iSrcWidth;
  cinfo.image_height = iSrcHeight;
  cinfo.input_components = 3;
  cinfo.in_color_space = JCS_YCbCr;
  
  jpeg_set_defaults(&cinfo);
  cinfo.raw_data_in = TRUE;
  jpeg_set_colorspace(&cinfo, JCS_YCbCr);

  cinfo.comp_info[0].h_samp_factor = 2;
  cinfo.comp_info[0].v_samp_factor = 2;
  cinfo.comp_info[1].h_samp_factor = 1;
  cinfo.comp_info[1].v_samp_factor = 1;
  cinfo.comp_info[2].h_samp_factor = 1;
  cinfo.comp_info[2].v_samp_factor = 1;

  jpeg_start_compress(&cinfo, TRUE);

  for (int i=0; i<ceil(iSrcHeight/16.0); i++)
    {
      for (int j=0; j<16; j++)
      {
	y_row_pointer[j] = y;
	y+=iSrcWidth;
	if (j%2 == 0)
	  {
	    u_row_pointer[j/2] = u;
	    v_row_pointer[j/2] = v;
	    u+=iSrcWidth/2;
	    v+=iSrcWidth/2;
	  }
      }
      component_pointer[0]=y_row_pointer;
      component_pointer[1]=u_row_pointer;
      component_pointer[2]=v_row_pointer;
      jpeg_write_raw_data(&cinfo,component_pointer,16);
    }
  jpeg_finish_compress(&cinfo);
  fclose(outfile);
  jpeg_destroy_compress(&cinfo);
  //delete []pcYbuffer;
}



