#include <new.h>
#include <signal.h>
#include <string.h> //for memset
#include <sys/time.h>
#include "mpeg_decoder.h"

extern void finishup(int enc_mode);
extern void Fatal(const char *fmt,...);

MPEG_Decoder::MPEG_Decoder(char* filename) : Decoder(filename)
{
  //printf("MPEG_Decoder: Entered constructor\n");
  int result = mpeg3_check_sig(filename);
  //printf("MPEG_Decoder: checked sig\n");
  if (!result)
    Fatal("%s not a valid MPEG file", filename);

  int error_return = 0;
  mpegfile = mpeg3_open(filename, &error_return);
  //printf("MPEG_Decoder: opened file\n");
  if (!mpegfile)
    Fatal("Failed to open MPEG file: %s", filename);

  int width = mpeg3_video_width(mpegfile, 0);
  int height = mpeg3_video_height(mpegfile, 0);
  int length = 0;//mpeg3_video_frames(mpegfile, 0);
  float framerate = mpeg3_frame_rate(mpegfile, 0);
  int channels = mpeg3_audio_channels(mpegfile, 0);
  int samplerate = mpeg3_sample_rate(mpegfile, 0);
  //printf("MPEG_Decoder: got properties\n");
  vidprops = new VideoProperties(width, height, length, framerate, channels,
				 samplerate);
  //printf("MPEG_Decoder: all done\n");
  frameno=0;
}

MPEG_Decoder::~MPEG_Decoder()
{
  delete vidprops;
  if (currframe)
    delete currframe;
  mpeg3_close(mpegfile);
}

VideoProperties* MPEG_Decoder::GetVideoProperties()
{
  return vidprops;
}

void timer_handler (int signum) 
{ 
  finishup(1);
}

int MPEG_Decoder::GetFrame(frame* &vidframe)
{
  //set up a timeout alarm
  struct sigaction sa; 
  struct itimerval timer; 
  memset (&sa, 0, sizeof (sa)); 
  sa.sa_handler = &timer_handler; 
  sigaction (SIGALRM, &sa, NULL); 
  /* Configure the timer to expire after 1sec... */ 
  timer.it_value.tv_sec = 1; 
  timer.it_value.tv_usec = 0; 
  timer.it_interval.tv_sec = 0; 
  timer.it_interval.tv_usec = 0; 

  setitimer (ITIMER_REAL, &timer, NULL);

  if (currframe)
    {
      delete currframe;
      currframe = 0;
    }

  int width = vidprops->GetWidth();
  int height = vidprops->GetHeight();

  char *ybuffer, *ubuffer, *vbuffer;
  ybuffer = new char[width*height];
  ubuffer = new char[width*height/4];
  vbuffer = new char[width*height/4];
  
  int result = mpeg3_read_yuvframe(mpegfile, ybuffer, ubuffer, vbuffer, 0, 0, 
				   width, height, 0);
  double timestamp = mpeg3_get_time(mpegfile);
  currframe = new frame(timestamp, ybuffer, ubuffer, vbuffer, width, height, 
			(width * height * 3) /2, 0, frameno);

  delete [] ybuffer;
  delete [] ubuffer;
  delete [] vbuffer;

  vidframe = currframe;

  frameno++;
  
  //We're done, so cancel the timeout
  timer.it_value.tv_sec = 0;
  setitimer (ITIMER_REAL, &timer, NULL);
 
  return result;
}

int MPEG_Decoder::GetAudio(short* buffer, int samples, int channel)
{
  int result;
  if (!channel)
    result = mpeg3_read_audio(mpegfile, NULL, buffer, channel, samples, 0);
  else
    result = mpeg3_reread_audio(mpegfile, NULL, buffer, channel, samples, 0);

  return result;
}

/*
double MPEG_Decoder::GetPercentageDone()
{
  return mpeg3_tell_percentage(mpegfile);
}
*/


int MPEG_Decoder::EndOfVideo()
{
  return mpeg3_end_of_video(mpegfile, 0);
}

long MPEG_Decoder::FrameNumber()
{
  return mpeg3_get_frame(mpegfile, 0);      
}
